@@ -169,7 +169,7 @@ from ae.i18n import get_f_string
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action   # type: ignore
 
 
-__version__ = '0.1.21'
+__version__ = '0.1.22'
 
 
 FLOW_PATH_ROOT_ID = 'ROOT'  #: pseudo item id needed for flow path jumper and for drop onto leave item button
@@ -180,12 +180,17 @@ FOCUS_FLOW_PREFIX = "->"    #: prefix shown in front of flow key of focused item
 INVALID_ITEM_ID_PREFIX_CHARS = '[' + '{'  #: invalid initial chars in item id (for to detect id | literal in flow key)
 # currently only the '[' char is used (for to put the node list data as literal in a flow key - see
 
-NODE_FILE_PREFIX = 'node_'  #: file name prefix for node imports/exports
-NODE_FILE_EXT = '.txt'      #: file extension for node imports/exports
+NODE_FILE_PREFIX = 'node_'                      #: file name prefix for node imports/exports
+NODE_FILE_EXT = '.txt'                          #: file extension for node imports/exports
 
+IMPORT_NODE_MAX_FILE_LEN = 8192                 #: maximum file length of importable <file.NODE_FILE_EXT> file
+IMPORT_NODE_MAX_ITEMS = 12                      #: maximum number of items to import or paste from clipboard
 
-LiszItem = Dict[str, Any]   #: node item data (nid) type
-LiszNode = List[LiszItem]   #: node/list type
+LiszItem = Dict[str, Any]                       #: node item data (nid) type
+LiszNode = List[LiszItem]                       #: node/list type
+
+NodeFileInfo = Tuple[str, LiszNode, str, str]   #: tuple of (node-parent-name, node-list, file_path, error-message)
+NodeFilesInfo = List[NodeFileInfo]              #: list of node file info tuples
 
 
 def check_item_id(item_id: str) -> str:
@@ -220,6 +225,31 @@ def correct_item_id(item_id: str) -> str:
     return item_id.lstrip(INVALID_ITEM_ID_PREFIX_CHARS)
 
 
+def node_from_literal(lit: str) -> LiszNode:
+    """ parse passed string for to integrate/add them into the current displayed node or item.
+
+    :param lit:             item/node data in string representation format.
+    :return:                LiszNode list of parsed LiszItem items.
+    """
+    if lit.startswith('['):
+        try:
+            node = ast.literal_eval(lit)
+        except (SyntaxError, ValueError):
+            node = list()
+
+    elif lit.startswith('{'):
+        try:
+            node = [ast.literal_eval(lit)]
+        except (SyntaxError, ValueError):
+            node = list()
+
+    else:
+        node = [dict(id=correct_item_id(item_id)) for item_id in norm_line_sep(lit).split('\n')
+                if correct_item_id(item_id)]
+
+    return node
+
+
 def item_sel_filter(item: LiszItem) -> bool:
     """ callable for to filter selected LiszItems.
 
@@ -327,22 +357,6 @@ class LiszDataMixin(ABC):
             elif set_sel_to:
                 item['sel'] = 1.0
 
-    @staticmethod
-    def current_item_or_node_from_literal(lit: str) -> LiszNode:
-        """ parse passed string for to integrate/add them into the current displayed node or item.
-
-        :param lit:             item/node data in string representation format.
-        :return:                LiszNode list of parsed LiszItem items.
-        """
-        if lit.startswith('['):
-            items = ast.literal_eval(lit)
-        elif lit.startswith('{'):
-            items = [ast.literal_eval(lit)]
-        else:
-            items = [dict(id=item_id) for item_id in norm_line_sep(lit).split('\n')
-                     if correct_item_id(item_id)]
-        return items
-
     def current_item_or_node_literal(self) -> str:
         """ return the currently focused/displayed item or node as repr string.
 
@@ -372,12 +386,15 @@ class LiszDataMixin(ABC):
             assert nid in self.current_node_items, f"DEL item data: {nid} not in {self.current_node_items}"
             self.current_node_items.remove(nid)
 
-    def edit_validate(self, old_idx: int, new_id: Optional[str] = None, want_node: Optional[bool] = None) -> str:
+    def edit_validate(self, old_item_index: int, new_id: Optional[str] = None, want_node: Optional[bool] = None,
+                      parent_node: Optional[LiszNode] = None, new_item_index: int = 0) -> str:
         """ validate the user changes after adding a new item or editing an existing item.
 
-        :param old_idx:             index in the current node of the edited item or -1 if a new item (to be added).
+        :param old_item_index:      index in the current node of the edited item or -1 if a new item (to be added).
         :param new_id:              new/edited id string.
         :param want_node:           True if the new/edited item will have a sub-node, False if not.
+        :param parent_node:         node where the edited/added item as to be updated/inserted (default=current list).
+        :param new_item_index:      index where the new item have to be inserted (default=0, ignored in edit item mode).
         :return:                    empty string on successful edit validation or on cancellation of new item (with
                                     empty id string), else error string or
                                     `'request_delete_confirmation_for_item'` if the user has to confirm the deletion
@@ -385,7 +402,10 @@ class LiszDataMixin(ABC):
                                     `'request_delete_confirmation_for_node'` if the user has to confirm the
                                     removal of the sub-node.
         """
-        add_item = old_idx == -1
+        add_item = old_item_index == -1
+        if parent_node is None:
+            parent_node = self.current_node_items
+
         if new_id is not None:
             if not new_id:
                 # on empty id cancel addition (if add_item), else request confirmation from user for item deletion
@@ -398,8 +418,8 @@ class LiszDataMixin(ABC):
                 if chk_err:
                     return chk_err
 
-            new_idx = self.find_item_index(new_id)
-            if new_idx != -1 and (add_item or new_idx != old_idx):
+            found_item_index = self.find_item_index(new_id, searched_node=parent_node)
+            if found_item_index != -1 and (add_item or found_item_index != old_item_index):
                 return get_f_string("item id '{new_id}' exists already")
 
         if add_item:                        # add new item
@@ -408,10 +428,10 @@ class LiszDataMixin(ABC):
             nid = dict(id=new_id)
             if want_node:
                 nid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
-            self.current_node_items.insert(0, nid)
+            parent_node.insert(new_item_index, nid)
 
         else:                               # edit item
-            nid = self.current_node_items[old_idx]
+            nid = parent_node[old_item_index]
             if new_id:
                 nid['id'] = new_id
             if want_node is not None and want_node != ('node' in nid):  # NOTE: != has lower priority than in operator
@@ -517,17 +537,17 @@ class LiszDataMixin(ABC):
         """
         if flow_path is None:
             flow_path = self.flow_path
-        current_node_items = self.root_node
+        node = self.root_node
         for flow_id in flow_path:
             if flow_action(flow_id) == 'enter':
                 node_id = flow_key(flow_id)
-                item = self.item_by_id(node_id, searched_node=current_node_items)
+                item = self.item_by_id(node_id, searched_node=node)
                 if 'node' not in item:
                     if strict:
-                        current_node_items = list()
+                        node = list()
                     break                           # actually not needed, will repair broken flow path
-                current_node_items = item['node']
-        return current_node_items
+                node = item['node']
+        return node
 
     def flow_path_quick_jump_nodes(self) -> List[str]:
         """ determine nodes relative to the current flow path to quick-jump to from current node.
@@ -605,31 +625,91 @@ class LiszDataMixin(ABC):
                 idx = min(max(-1, delta), 0)
             self.change_flow(id_of_flow('focus', 'item', self.current_node_items[filtered_indexes[idx]]['id']))
 
-    def import_nodes(self, file_path: str = ".") -> List[str]:
-        """ import all nodes found in the documents app folder of this app into the currently displayed node.
+    def importable_node_files(self, folder_path: str = ".") -> NodeFilesInfo:
+        """ load and check all nodes found in the documents app folder of this app.
 
-        :param file_path:   path to the folder from where the node data get imported (def=current working directory).
-        :return:            empty list if found node(s) got imported without errors,
-                            else the list of error strings prefixed with the node file path.
+        :param folder_path: path to the folder where the node files are situated (def=current working directory).
+        :return:            list of node file tuples of (node_name, node, file_path, error-message).
         """
-        errors = list()
-        try:
-            for node_file in os.scandir(file_path):
-                file_name = node_file.name
-                if file_name.endswith(NODE_FILE_EXT) and node_file.is_file():
-                    with open(node_file.path) as file_handle:
-                        file_content = file_handle.read()
+        node_file_info_tuples = list()
+        for node_file in os.scandir(folder_path):
+            file_name = node_file.name
+            if file_name.endswith(NODE_FILE_EXT) and node_file.is_file():
+                node_file_info_tuples.append(self.import_file_info(node_file.path))
+        return node_file_info_tuples
 
-                    node_name = "imported node " + file_name[len(NODE_FILE_PREFIX):-len(NODE_FILE_EXT)]
-                    err_msg = self.edit_validate(-1, node_name, want_node=True)
-                    if err_msg:
-                        errors.append(node_file.path + ":" + err_msg)
-                    else:
-                        # update the list instance that got already added by edit_validate() with the loaded node data
-                        self.current_node_items[0]['node'].extend(ast.literal_eval(file_content))
-        except (FileExistsError, FileNotFoundError, OSError, ValueError) as ex:
-            errors.append(str(ex))
-        return errors
+    @staticmethod
+    def import_file_info(file_path: str, node_name: str = "") -> NodeFileInfo:
+        """ load node file and determine node content.
+
+        :param file_path:   path to the node file to import.
+        :param node_name:   optional name/id of the parent node name. If not passed then it will be determined
+                            from the file name (removing :data:`NODE_FILE_PREFIX` and file extension).
+        :return:            node file info tuple as: (node name, node-data or empty list, file path, error message).
+        """
+        if not node_name:
+            node_name = os.path.splitext(os.path.basename(file_path))[0]
+            if node_name.startswith(NODE_FILE_PREFIX):
+                node_name = node_name[len(NODE_FILE_PREFIX):]
+
+        try:
+            with open(file_path) as file_handle:
+                file_content = file_handle.read()
+        except (FileExistsError, FileNotFoundError, OSError) as ex:
+            return node_name, list(), file_path, f"file load error {ex}"
+
+        file_len = len(file_content)
+        if file_len > IMPORT_NODE_MAX_FILE_LEN:
+            return (node_name, list(), file_path,
+                    f"import file is bigger than {IMPORT_NODE_MAX_FILE_LEN} bytes ({file_len})")
+
+        node = node_from_literal(file_content)
+        if not node:
+            return node_name, list(), file_path, "invalid file content or empty file"
+
+        if len(node) > IMPORT_NODE_MAX_ITEMS:
+            return (node_name, list(), file_path,
+                    f"import file contains more than {IMPORT_NODE_MAX_ITEMS} items ({len(node)})")
+
+        return node_name, node, file_path, ""
+
+    def import_items(self, node: LiszNode, parent: Optional[LiszNode] = None, item_index: int = 0) -> str:
+        """ import passed node items into the passed parent/destination node at the given index.
+
+        :param node:            node with items to import/add.
+        :param parent:          destination node to add the node items to (def=current node list).
+        :param item_index:      list index in the destination node where the items have to be inserted (default=0).
+        :return:                empty string if all items of node got imported correctly, else error message string.
+        """
+        error_messages = list()
+        for item in node:
+            err_msg = self.edit_validate(-1, item['id'], parent_node=parent, new_item_index=item_index)
+            if err_msg:
+                error_messages.append(err_msg)
+            else:
+                item_index += 1
+
+        return "\n".join(error_messages)
+
+    def import_node(self, node_id: str, node: LiszNode, parent: Optional[LiszNode] = None, item_index: int = 0) -> str:
+        """ import passed node as new node into the passed parent node at the given index.
+
+        :param node_id:         id of the new node to import/add.
+        :param node:            node with items to import/add.
+        :param parent:          destination node to add the new node to (def=current node list).
+        :param item_index:      list index in the parent node where the items have to be inserted (default=0).
+        :return:                empty string if node got added/imported correctly, else error message string.
+        """
+        if parent is None:
+            parent = self.current_node_items
+
+        err_msg = self.edit_validate(-1, node_id, want_node=True, parent_node=parent, new_item_index=item_index)
+        if not err_msg:
+            # extend the list instance (that got already created/added by edit_validate()) with the loaded node data
+            # use self.import_items() to ensure correct node ids, instead of: parent[item_index]['node'].extend(node)
+            err_msg = self.import_items(node, parent=parent[item_index]['node'])
+
+        return err_msg
 
     def item_by_id(self, item_id: str, searched_node: Optional[LiszNode] = None) -> LiszItem:
         """ search item in either the passed or the current node.
@@ -676,10 +756,10 @@ class LiszDataMixin(ABC):
 
         return True
 
-    def node_info(self, flow_path: List[str], what: Tuple[str, ...] = ()) -> Dict[str, Union[int, str, List[str]]]:
+    def node_info(self, node: LiszNode, what: Tuple[str, ...] = ()) -> Dict[str, Union[int, str, List[str]]]:
         """ determine statistics info for the node specified by :paramref:`~node_info.flow_path`.
 
-        :param flow_path:       flow path to identify node (pass empty list to get the info of the root node).
+        :param node:            node to get info for.
         :param what:            pass tuple of statistic info fields for to include only these into the returned dict
                                 (passing an empty tuple or nothing will include all the following fields):
 
@@ -688,7 +768,6 @@ class LiszDataMixin(ABC):
                                 * 'node_count': number of sub-nodes.
                                 * 'selected_leaf_count': number of selected sub-leaves.
                                 * 'unselected_leaf_count': number of unselected sub-leaves.
-                                * 'name': name of the item containing this node.
                                 * 'names': list of all sub-item/-node names/ids.
                                 * 'leaf_names': list of all sub-leaf names.
                                 * 'selected_leaf_names': list of all selected sub-leaf names.
@@ -696,20 +775,13 @@ class LiszDataMixin(ABC):
 
         :return:                dict with the node info specified by the :paramref:`~node_info.what` argument.
         """
-        if flow_path:
-            parent_node = self.flow_path_node(flow_path[:-1], strict=True)
-            name = flow_key(flow_path[-1])
-        else:
-            parent_node = self.root_node
-            name = FLOW_PATH_ROOT_ID
-
-        names = self.sub_item_ids(node=parent_node, item_id=name, leaves_only=False)
+        names = self.sub_item_ids(node=node, leaves_only=False)
         count = len(names)
-        leaf_names = self.sub_item_ids(node=parent_node, item_id=name)
+        leaf_names = self.sub_item_ids(node=node)
         leaf_count = len(leaf_names)
-        selected_leaf_names = self.sub_item_ids(node=parent_node, item_id=name, hide_sel_val=False)
+        selected_leaf_names = self.sub_item_ids(node=node, hide_sel_val=False)
         selected_leaf_count = len(selected_leaf_names)          # noqa: F841
-        unselected_leaf_names = self.sub_item_ids(node=parent_node, item_id=name, hide_sel_val=True)
+        unselected_leaf_names = self.sub_item_ids(node=node, hide_sel_val=True)
         unselected_leaf_count = len(unselected_leaf_names)      # noqa: F841
         node_count = count - leaf_count                         # noqa: F841
 
